@@ -13,6 +13,7 @@
 
 #include <QAudio>
 #include <QDir>
+#include <QGuiApplication>
 
 #if defined Q_OS_WIN
 
@@ -22,6 +23,7 @@ typedef SSIZE_T ssize_t;
 #endif
 
 #include <vlc/vlc.h>
+#include <vlc/libvlc_version.h>
 
 #include "config-upnp-qt.h"
 
@@ -44,7 +46,7 @@ public:
 
     qint64 mMediaDuration = 0;
 
-    QMediaPlayer::State mPreviousPlayerState = QMediaPlayer::StoppedState;
+    QMediaPlayer::PlaybackState mPreviousPlayerState = QMediaPlayer::StoppedState;
 
     QMediaPlayer::MediaStatus mPreviousMediaStatus = QMediaPlayer::NoMedia;
 
@@ -68,7 +70,7 @@ public:
 
     void mediaIsEnded();
 
-    bool signalPlaybackChange(QMediaPlayer::State newPlayerState);
+    bool signalPlaybackChange(QMediaPlayer::PlaybackState newPlayerState);
 
     void signalMediaStatusChange(QMediaPlayer::MediaStatus newMediaStatus);
 
@@ -95,7 +97,7 @@ AudioWrapper::AudioWrapper(QObject *parent) : QObject(parent), d(std::make_uniqu
 {
     d->mParent = this;
     d->mInstance = libvlc_new(0, nullptr);
-    libvlc_set_user_agent(d->mInstance, "elisa", "Elisa Music Player");
+    libvlc_set_user_agent(d->mInstance, QGuiApplication::applicationDisplayName().toUtf8().constData(), "Elisa Music Player");
     libvlc_set_app_id(d->mInstance, "org.kde.elisa", ELISA_VERSION_STRING, "elisa");
 
     d->mPlayer = libvlc_media_player_new(d->mInstance);
@@ -112,7 +114,11 @@ AudioWrapper::AudioWrapper(QObject *parent) : QObject(parent), d(std::make_uniqu
     libvlc_event_attach(d->mPlayerEventManager, libvlc_MediaPlayerPlaying, &vlc_callback, d.get());
     libvlc_event_attach(d->mPlayerEventManager, libvlc_MediaPlayerPaused, &vlc_callback, d.get());
     libvlc_event_attach(d->mPlayerEventManager, libvlc_MediaPlayerStopped, &vlc_callback, d.get());
+#if LIBVLC_VERSION_MAJOR >= 4
+    libvlc_event_attach(d->mPlayerEventManager, libvlc_MediaPlayerStopping, &vlc_callback, d.get());
+#else
     libvlc_event_attach(d->mPlayerEventManager, libvlc_MediaPlayerEndReached, &vlc_callback, d.get());
+#endif
     libvlc_event_attach(d->mPlayerEventManager, libvlc_MediaPlayerEncounteredError, &vlc_callback, d.get());
     libvlc_event_attach(d->mPlayerEventManager, libvlc_MediaPlayerPositionChanged, &vlc_callback, d.get());
     libvlc_event_attach(d->mPlayerEventManager, libvlc_MediaPlayerSeekableChanged, &vlc_callback, d.get());
@@ -126,6 +132,14 @@ AudioWrapper::AudioWrapper(QObject *parent) : QObject(parent), d(std::make_uniqu
 AudioWrapper::~AudioWrapper()
 {
     if (d->mInstance) {
+        d->mPowerInterface.setPreventSleep(false);
+        if (d->mPlayer && d->mPreviousPlayerState != QMediaPlayer::StoppedState) {
+#if LIBVLC_VERSION_MAJOR >= 4
+            libvlc_media_player_stop_async(d->mPlayer);
+#else
+            libvlc_media_player_stop(d->mPlayer);
+#endif
+        }
         libvlc_release(d->mInstance);
     }
 }
@@ -184,7 +198,7 @@ bool AudioWrapper::seekable() const
     return d->mIsSeekable;
 }
 
-QMediaPlayer::State AudioWrapper::playbackState() const
+QMediaPlayer::PlaybackState AudioWrapper::playbackState() const
 {
     return d->mPreviousPlayerState;
 }
@@ -217,10 +231,18 @@ void AudioWrapper::setSource(const QUrl &source)
 {
     if (source.isLocalFile()) {
         qCDebug(orgKdeElisaPlayerVlc) << "AudioWrapper::setSource reading local resource";
+#if LIBVLC_VERSION_MAJOR >= 4
+        d->mMedia = libvlc_media_new_path(QDir::toNativeSeparators(source.toLocalFile()).toUtf8().constData());
+#else
         d->mMedia = libvlc_media_new_path(d->mInstance, QDir::toNativeSeparators(source.toLocalFile()).toUtf8().constData());
+#endif
     } else {
         qCDebug(orgKdeElisaPlayerVlc) << "AudioWrapper::setSource reading remote resource";
+#if LIBVLC_VERSION_MAJOR >= 4
+        d->mMedia = libvlc_media_new_location(source.url().toUtf8().constData());
+#else
         d->mMedia = libvlc_media_new_location(d->mInstance, source.url().toUtf8().constData());
+#endif
     }
 
     if (!d->mMedia) {
@@ -229,7 +251,11 @@ void AudioWrapper::setSource(const QUrl &source)
                  << libvlc_errmsg()
                  << QDir::toNativeSeparators(source.toLocalFile()).toUtf8().constData();
 
+#if LIBVLC_VERSION_MAJOR >= 4
+        d->mMedia = libvlc_media_new_path(QDir::toNativeSeparators(source.toLocalFile()).toLatin1().constData());
+#else
         d->mMedia = libvlc_media_new_path(d->mInstance, QDir::toNativeSeparators(source.toLocalFile()).toLatin1().constData());
+#endif
         if (!d->mMedia) {
             qCDebug(orgKdeElisaPlayerVlc) << "AudioWrapper::setSource"
                      << "failed creating media"
@@ -239,6 +265,13 @@ void AudioWrapper::setSource(const QUrl &source)
         }
     }
 
+    // By default, libvlc caches only next 1000 (ms, 0..60000) of the playback,
+    // which is unreasonable given our usecase of sequential playback.
+    libvlc_media_add_option(d->mMedia, ":file-caching=10000");
+    libvlc_media_add_option(d->mMedia, ":live-caching=10000");
+    libvlc_media_add_option(d->mMedia, ":disc-caching=10000");
+    libvlc_media_add_option(d->mMedia, ":network-caching=10000");
+
     libvlc_media_player_set_media(d->mPlayer, d->mMedia);
 
     if (d->signalPlaybackChange(QMediaPlayer::StoppedState)) {
@@ -248,6 +281,8 @@ void AudioWrapper::setSource(const QUrl &source)
     d->signalMediaStatusChange(QMediaPlayer::LoadingMedia);
     d->signalMediaStatusChange(QMediaPlayer::LoadedMedia);
     d->signalMediaStatusChange(QMediaPlayer::BufferedMedia);
+
+    d->mHasSavedPosition = false;
 }
 
 void AudioWrapper::setPosition(qint64 position)
@@ -261,7 +296,11 @@ void AudioWrapper::setPosition(qint64 position)
         return;
     }
 
+#if LIBVLC_VERSION_MAJOR >= 4
+    libvlc_media_player_set_position(d->mPlayer, static_cast<float>(position) / d->mMediaDuration, true);
+#else
     libvlc_media_player_set_position(d->mPlayer, static_cast<float>(position) / d->mMediaDuration);
+#endif
 }
 
 void AudioWrapper::savePosition(qint64 position)
@@ -308,7 +347,11 @@ void AudioWrapper::stop()
         return;
     }
 
+#if LIBVLC_VERSION_MAJOR >= 4
+    libvlc_media_player_stop_async(d->mPlayer);
+#else
     libvlc_media_player_stop(d->mPlayer);
+#endif
 }
 
 void AudioWrapper::seek(qint64 position)
@@ -332,7 +375,7 @@ void AudioWrapper::playerVolumeChanged()
 {
 }
 
-void AudioWrapper::playerStateSignalChanges(QMediaPlayer::State newState)
+void AudioWrapper::playerStateSignalChanges(QMediaPlayer::PlaybackState newState)
 {
     QMetaObject::invokeMethod(this, [this, newState]() {
         Q_EMIT playbackStateChanged(newState);
@@ -415,8 +458,15 @@ void AudioWrapperPrivate::vlcEventCallback(const struct libvlc_event_t *p_event)
         qCDebug(orgKdeElisaPlayerVlc) << "AudioWrapperPrivate::vlcEventCallback" << "libvlc_MediaPlayerStopped";
         signalPlaybackChange(QMediaPlayer::StoppedState);
         break;
+#if LIBVLC_VERSION_MAJOR >= 4
+    case libvlc_MediaPlayerStopping:
+        qCDebug(orgKdeElisaPlayerVlc) << "AudioWrapperPrivate::vlcEventCallback"
+                                      << "libvlc_MediaPlayerStopping";
+#else
     case libvlc_MediaPlayerEndReached:
-        qCDebug(orgKdeElisaPlayerVlc) << "AudioWrapperPrivate::vlcEventCallback" << "libvlc_MediaPlayerEndReached";
+        qCDebug(orgKdeElisaPlayerVlc) << "AudioWrapperPrivate::vlcEventCallback"
+                                      << "libvlc_MediaPlayerEndReached";
+#endif
         signalMediaStatusChange(QMediaPlayer::BufferedMedia);
         signalMediaStatusChange(QMediaPlayer::NoMedia);
         signalMediaStatusChange(QMediaPlayer::EndOfMedia);
@@ -471,7 +521,7 @@ void AudioWrapperPrivate::mediaIsEnded()
     mMedia = nullptr;
 }
 
-bool AudioWrapperPrivate::signalPlaybackChange(QMediaPlayer::State newPlayerState)
+bool AudioWrapperPrivate::signalPlaybackChange(QMediaPlayer::PlaybackState newPlayerState)
 {
     if (mPreviousPlayerState != newPlayerState) {
         mPreviousPlayerState = newPlayerState;
