@@ -15,9 +15,35 @@
 
 #include <QAbstractProxyModel>
 #include <QMediaPlayer>
+#include <QMimeType>
+#include <QFileInfo>
 
 #include <memory>
 
+class M3uPlaylistParser
+{
+public:
+    QList<QUrl> fromPlaylist(const QUrl &fileName, const QByteArray &fileContent);
+    QString toPlaylist(const QUrl &fileName, const QList<QString> &listOfUrls);
+};
+
+class PlsPlaylistParser
+{
+public:
+    QList<QUrl> fromPlaylist(const QUrl &fileName, const QByteArray &fileContent);
+    QString toPlaylist(const QUrl &fileName, const QList<QString> &listOfUrls);
+};
+
+class ELISALIB_EXPORT PlaylistParser
+{
+public:
+    QList<QUrl> fromPlaylist(const QUrl &fileName, const QByteArray &fileContent);
+    QString toPlaylist(const QUrl &fileName, const QList<QString> &listOfUrls);
+
+private:
+    int filterImported(QList<QUrl>& result, const QUrl &playlistUrl);
+};
+
 class MediaPlayList;
 class MediaPlayListProxyModelPrivate;
 
@@ -75,6 +101,15 @@ private:
                READ tracksCount
                NOTIFY tracksCountChanged)
 
+    Q_PROPERTY(bool partiallyLoaded
+               READ partiallyLoaded
+               NOTIFY partiallyLoadedChanged
+               RESET resetPartiallyLoaded)
+
+    Q_PROPERTY(bool canOpenLoadedPlaylist
+               READ canOpenLoadedPlaylist
+               NOTIFY canOpenLoadedPlaylistChanged)
+
 public:
 
     explicit MediaPlayListProxyModel(QObject *parent = nullptr);
@@ -123,6 +158,10 @@ public:
 
     [[nodiscard]] QVariantMap persistentState() const;
 
+    [[nodiscard]] bool partiallyLoaded() const;
+
+    [[nodiscard]] bool canOpenLoadedPlaylist() const;
+
     int mSeekToBeginningDelay = 2000;
 
 
@@ -167,12 +206,13 @@ public Q_SLOTS:
 
     void loadPlayList(const QUrl &fileName);
 
+    void loadPlayList(const QUrl &fileName, ElisaUtils::PlayListEnqueueMode enqueueMode, ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
+
     void setPersistentState(const QVariantMap &persistentState);
 
-    void enqueueDirectory(const QUrl &fileName,
-                          ElisaUtils::PlayListEntryType databaseIdType,
-                          ElisaUtils::PlayListEnqueueMode enqueueMode,
-                          ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay, int depth);
+    void openLoadedPlayList();
+
+    void resetPartiallyLoaded();
 
 Q_SIGNALS:
 
@@ -218,6 +258,10 @@ Q_SIGNALS:
 
     void seek(qint64 position);
 
+    void partiallyLoadedChanged();
+
+    void canOpenLoadedPlaylistChanged();
+
 private Q_SLOTS:
 
     void sourceRowsAboutToBeInserted(const QModelIndex &parent, int start, int end);
@@ -254,6 +298,14 @@ private:
 
     void determineAndNotifyPreviousAndNextTracks();
 
+    void loadLocalFile(DataTypes::EntryDataList &newTracks, QSet<QString> &processedFiles, const QFileInfo &fileInfo);
+
+    void loadLocalPlayList(DataTypes::EntryDataList &newTracks, QSet<QString> &processedUFiles, const QUrl &fileName, const QByteArray &fileContent);
+
+    void loadLocalDirectory(DataTypes::EntryDataList &newTracks, QSet<QString> &processedFiles, const QUrl &dirName);
+
+    int filterLocalPlayList(QList<QUrl>& result, const QUrl &playlistUrl);
+
     std::unique_ptr<MediaPlayListProxyModelPrivate> d;
 };
 
